var jsonpAdclient = null;

function jsonp_callback(data) {
  var responseDoc = null;
  try {
    responseDoc = new DOMParser().parseFromString(data["contents"], 'text/xml');
  } catch(e) {
    Putpat.reportError(e, "unknown", "ad_client.js", 7);
  }

  try {
    if (jsonpAdclient != null) {
      jsonpAdclient.jsonpLoadedCallback(responseDoc);
    }
  } catch(e) {
    Putpat.reportError(e, "unknown", "ad_client.js", 16);
  }
}

var Adclient = function(initDoneCallback){
  var clipsBetweenAds = 10000;
  var clipsBeforeFirstAd = 10000;
  var deviceData = null;
  var startUrls = [];
  var self = {};
  var jsonpCallback = null;
  var previousChannel = {};
  var previousClip = {};

  init(initDoneCallback);

  function init() {
    var params = {};

    params["client"] = CLIENT_STRING;
    params["player_id"] = Putpat.getPlayerId();

    Putpat.callApi("Marketing2.init", params, function(data) {
      try {
        if (data.device != null) {
          clipsBetweenAds = data.device.clip_count;
          clipsBeforeFirstAd = data.device.start_clip_count;
          startUrls = data.device.on_start;

          if (clipsBetweenAds == null || clipsBetweenAds < 1) {
            clipsBetweenAds = 10000;
          }
          if (clipsBeforeFirstAd == null || clipsBeforeFirstAd < 1) {
            clipsBeforeFirstAd = 10000;
          }
        }

        if (initDoneCallback != null) {
          initDoneCallback();
        }
      } catch(e) {
        Putpat.reportError(e, "unknown", "ad_client.js", 56);
      }

    }, true);
  }

  function loadVastUrlsForAdvertisement(adtype, callback) {
    Putpat.debug("Loading Ads");

    Channels.getCurrentClipForChannel(ChannelView.currentChannel, function(clip) {
      try {
        var params = {
          "adType": adtype,
          "client": CLIENT_STRING,
          "player_id": Putpat.getPlayerId(),
          "next_channel_id": ChannelView.currentChannel.id,
          "previous_channel_id": previousChannel.id,
          "next_asset_id": clip.asset_id,
          "previous_asset_id": previousClip.asset_id,
          "device_id": Utils.base64("user"+Putpat.user["id"])
        };
        previousChannel = ChannelView.currentChannel;
        previousClip = clip;

        Putpat.reportAction("AD", "PUTPAT_VASTURL_REQUESTED");
        Putpat.callApi("Marketing2.vastUrls", params, function(result) {
          callback(result.ad_config.ads);
        }, true);
      } catch(e) {
        Putpat.reportError(e, "unknown", "ad_client.js", 84);
      }
    });
  }

  self.jsonpLoadedCallback = function(data) {
    try {
      jsonpAdclient = null;
      Putpat.debug("JSONP geladen!");
      if (jsonpCallback) {
        Putpat.debug("callback!");
        if (data.firstChild) {
          jsonpCallback(data.firstChild);
          jsonpCallback = null;
        } else {
          jsonpCallback(null);
          jsonpCallback = null;
        }
      }
    } catch(e) {
      Putpat.reportError(e, "unknown", "ad_client.js", 104);
    }
  };

  function loadXmlForAdvertisement(url, callback) {
    if (Utils.isStringEmpty(url)) {
      callback(null);
      return;
    }

    url = url.replace("[timestamp]", Utils.randomString(10));
    url = url.replace("##random##", Utils.randomString(10));

    if (Device.advertViaJsonP == true) {
      Putpat.debug("lade jsonp " + url)
      jsonpAdclient = self;
      jsonpCallback = callback;
      // Sicherheitshalber einen Fallback einrichten.
      // Wenn nach 5 sek noch keine Antwort da is => ohne
      // Werbung weiter machen
      setTimeout(function() {
        if (jsonpAdclient != null) {
          Putpat.debug("timeout, no video was played within 5sec");
          jsonpAdclient = null;
          jsonpCallback = null;
          callback(null);
        }
      }, 5000);
      $("head").append("<script src='"+url+"' type='text/javascript'></script>");
    } else {
      XHRPool.getXHR(function(xhr) {
          xhr.onreadystatechange = function() {
             if(xhr.readyState == 4 && xhr.status == 200) {
               try {
                 var xmlData = xhr.responseXML;
                 callback(xmlData.firstChild);
                 XHRPool.done(xhr);
               } catch (e) {
                 Putpat.debug(e);
                 Putpat.debug("werbe fehler");
                 Putpat.reportError(e, "unknown", "ad_client.js", 144);
                 callback(null);
                 XHRPool.done(xhr);
               }
              } else if (xhr.readyState == 4) {
                callback(null);
                XHRPool.done(xhr);
              }
          };

          Putpat.debug("Hole werbung..." +  url);

          xhr.open("GET", url);
          xhr.setRequestHeader("Content-Type", "text/xml");
          xhr.send();
      });
    }
  }

  function getUrls(adtype, callback) {
    if (adtype == "onstart") {
      if (startUrls != null && startUrls.length > 0) {
        Tracking.trackStat("ad_backend_request");
      }
      callback(startUrls);
    } else {
        Tracking.trackStat("ad_backend_request");
        loadVastUrlsForAdvertisement(adtype, function(urls) {
           callback(urls);
        });
    }
  };

  function reportPutpatVastError(isFallback) {
    if (isFallback) {
      Putpat.reportAction("AD", "PUTPAT_VAST_ERROR_fallback");
    } else {
      Putpat.reportAction("AD", "PUTPAT_VAST_ERROR");
    }
  }

  function parserResult(advert, callback, trackings, isFallback) {
    try {
      if (advert == null) {
        Tracking.trackUrl(trackings.ad_response_error);
        callback("ERROR_3", null);
        reportPutpatVastError(isFallback);
      } else if (advert["duration"] == 1 || advert["url"] == null || advert["url"].length == 0) {
        Putpat.reportAction("AD", "PUTPAT_VAST_EMPTY");
        Tracking.trackUrl(advert.putpat_trackings.ad_response_empty);
        var urls = [];
        for (var i in advert.trackings) {
          if (advert.trackings[i].key == "impressions") {
            urls.push(advert.trackings[i].url);
          }
        }
        if (urls.length > 0) {
          Putpat.callUrls(urls, false, false);
        }
        callback("ERROR_EMPTY", null);
      } else {
        var video = self.getVideoClip(advert, "mp4", "progressive");
        if (video != null) {
          Tracking.trackUrl(advert.putpat_trackings.ad_response_success);
          advert.video_url = video;
          callback("OK", advert);
        } else {
          reportPutpatVastError(advert.fallback);
          Tracking.trackUrl(advert.putpat_trackings.ad_response_error);
          callback("ERROR_NO_VIDEO", null);
        }
      }
    } catch(e) {
      Putpat.reportError(e, "unknown", "ad_client.js", 217);
    }
  }

  function callXMLLoader(url, isFallback, callback) {
    if (url == null) {
      callback("Error77", null)
      return;
    }
    if (isFallback) {
      Putpat.reportAction("AD", "PUTPAT_VAST_REQUESTED_fallback");
    } else {
      Putpat.reportAction("AD", "PUTPAT_VAST_REQUESTED");
    }
    Putpat.debug("loading advert! " + JSON.stringify(url));
    Tracking.trackUrl(url.trackings.ad_requested);
    loadXmlForAdvertisement(url.url, function(data) {
        if (data == null) {
          try {
            Tracking.trackUrl(url.trackings.ad_response_error);
            reportPutpatVastError(isFallback);
            callback("Error2", null);
          } catch(e) {
            Putpat.reportError(e, "unknown", "ad_client.js", 240);
          }
        } else {

          if (data.nodeName == "VideoAdServingTemplate") {
            VastParser(loadXmlForAdvertisement).parseDom(data, function(advert){
              try {
                if (advert != null) {
                  advert.fallback = isFallback;
                  advert.putpat_trackings = url.trackings;
                }
                parserResult(advert, callback, url.trackings, isFallback);
              } catch(e) {
                Putpat.reportError(e, "unknown", "ad_client.js", 253);
              }
            });
          } else if (data.nodeName == "VAST") {
            Vast2Parser(loadXmlForAdvertisement).parseDom(data, function(advert) {
              try {
                if (advert != null) {
                  advert.fallback = isFallback;
                  advert.putpat_trackings = url.trackings;
                }
                parserResult(advert, callback, url.trackings, isFallback);
              } catch(e) {
                Putpat.reportError(e, "unknown", "ad_client.js", 265);
              }
            });
          } else if (data.nodeName == "content") {
            SmartclipParser().parseDom(data, function(advert) {
              try {
                if (advert != null) {
                  advert.fallback = isFallback;
                  advert.putpat_trackings = url.trackings;
                }
                parserResult(advert, callback, url.trackings, isFallback);
              } catch(e) {
                Putpat.reportError(e, "unknown", "ad_client.js", 277);
              }
            });
          } else {
            try {
              Tracking.trackUrl(url.trackings.ad_response_error);
              callback("Error15", null);
              reportPutpatVastError(isFallback);
            } catch(e) {
              Putpat.reportError(e, "unknown", "ad_client.js", 286);
            }
          }
        }
    });
  }

  self.loadAdvertisement = function(urls, callback) {
    try {
      if (urls != null && urls[0] != null && urls[0] != "null") {
        Putpat.debug(JSON.stringify(urls));
        callXMLLoader(urls[0]["default"], false, function(status, advert) {
          if ((status != "OK" || self.getVideoClip(advert, "mp4", "progressive") == null) && urls[0].fallback != null && !Utils.isStringEmpty(urls[0]["fallback"].url)){
            Putpat.debug("Nutze Fallback URL");
            callXMLLoader(urls[0]["fallback"], true, callback);
          } else {
            callback(status, advert);
          }
        });
      } else {
          callback("Error4", null);
      }
    } catch(e) {
      Putpat.reportError(e, "unkown", "ad_client#loadAdvertisement", "");
    }
  };

  self.getAdvertisement = function(adtype, callback) {
    try {
      getUrls(adtype, function(result){
        self.loadAdvertisement(result, callback);
      });
    } catch(e) {
      Putpat.reportError(e, "unkown", "ad_client#getAdvertisement", "");
    }
  };

  self.getVideoClip = function(advert, filetype, delivery) {
    try {
      if (advert == null || advert["url"] == null || advert["url"].length == 0) {
        Putpat.debug("keine urls in advert gefunden");
        return null;
      }

      urls = advert["url"];

      // Wenn nur eine URL angegeben ist, dann annehmen, das diese perfekt ist :)
      if (urls.length == 1) {
        return urls[0]["url"];
      }

      // Sort, highest bitrate first
      urls = urls.sort(function(a,b) {
        if (a["bitrate"] < b["bitrate"]) {
          return 1;
        } else if (a["bitrate"] > b["bitrate"]) {
          return -1;
        } else {
          return 0;
        }
      });

      // Erst schauen obs was mit delivery+type gibt
      for(i in urls) {
        if (urls[i]["delivery"] == delivery && urls[i]["type"].indexOf(filetype) >= 0) {
          return urls[i]["url"];
        }
      }

      // Wenn nicht, dann nur type
      for(i in urls) {
        if (urls[i]["type"].indexOf(filetype) >= 0) {
          return urls[i]["url"];
        }
      }

    } catch(e) {
      Putpat.reportError(e, "unkown", "ad_client#getVideoClip", "");
    }

    return null;
  }

  self.shouldPlayAdvert = function(videoCount, adsPlayed){
    var adBeginCount = clipsBetweenAds;
    try {
      if (adsPlayed == 0) {
        adBeginCount = clipsBeforeFirstAd;
      }
      Putpat.debug("===========> Clips before Ad:" + adBeginCount + " current count : " + videoCount );
    } catch(e) {
      Putpat.reportError(e, "unkown", "ad_client#shouldPlayAdvert", "");
    }

    return (videoCount >= adBeginCount);
  }

  return self;
}
