/**
 * Diese Klasse enthält das komplette Channel Handling,
 * inklusive VQ Channel, skippen, einzelnem Asset oder
 * Playlist abspielen.
 */
var Channels = function () {
    var channels = [];
    var videoData = [];
    var self = {};
    var currentVQ = null;

    /**
     * Ein JSON-Block normalisieren in ein
     * Hash, mit dem die App arbeitet
     */
    function parseClip(clip) {
      var item = {}

      item["asset_id"] = clip["asset"]["id"];
      item["title"] = clip["title"];
      item["url_low"] = clip["tokens"]["low"];
      item["url_medium"] = clip["tokens"]["medium"];
      item["url_high"] = clip["tokens"]["high"];
      item["url_preview"] = clip["tokens"]["preview"];
      item["clipInfo"] = clip;
      item["start_time"] = new Date(clip["absolute_start_time"] / 25 * 1000);
      item["start_offset"] = clip["asset"]["video_file"]["start_offset"] / 25;
      item["duration"] = clip["duration"] / 25;
      item["aspect_ratio"] = clip["asset"]["video_file"]["aspect_ratio"];
      item["video_file_id"] = clip["asset"]["video_file"]["id"];
      item["similar_artists"] = clip["similar_artists"];
      item["asset_type"] = clip["asset"]["type"];
      item["similar_artists"] = clip["similar_artists"];
      if (clip["asset"]["artist"] != null) {
        item["artist_id"] = clip["asset"]["artist"]["id"];
        item["artist"] = clip["asset"]["artist"]["title"];
        item["artist_picture_copyright"] = clip["asset"]["artist"]["poster_picture_credit"];
      }
      if (clip["asset"]["album"] != null) {
        item["album_id"] = clip["asset"]["album"]["id"];
        item["album_title"] = clip["asset"]["album"]["title"];
        item["album_release_year"] = clip["asset"]["album"]["release_year"];
      }

      videoData[item["url"]] = item;
      return item;
    }

    function loadClipsForVQ(vq, callback) {
      Putpat.debug("lade Clips!");
      Putpat.getVeequalizedClips(vq, function (data) {
        clipList = [];
        if (data != null) {
          for (var i in data) {
            if (data[i]["clip"]) {
              clipList.push(parseClip(data[i]["clip"]));
            }
          }
        };
        callback(clipList);
      });
    }

    function fetchVQClips(channel, callback) {
      if (currentVQ != null) {
        loadClipsForVQ(currentVQ, callback);
      } else {
        Putpat.getCurrentVeequalizer(function(vq) {
          loadClipsForVQ(vq, callback);
          currentVQ = vq;
        });
      }
    }

    function fetchChannelClips(channel, callback) {
      Putpat.debug("lade Clips!");
      Putpat.getClipsForChannel(channel, function (data) {
        result = JSON.parse(data);
        clipList = [];
        for (var i in result) {
          clipList.push(parseClip(result[i]["clip"]));
        }
        callback(clipList);
      });
    }

    function fetchClips(channel, callback) {
      if (channel["id"] == 0) {
        fetchVQClips(channel, callback);
      } else {
        fetchChannelClips(channel, callback);
      }
    }

    function findCurrentClipOffsetForChannelId(channel_id) {
      if (channels[channel_id] != null) {
        for (var i in channels[channel_id]) {
          var clip = channels[channel_id][i];
          if (clip["start_time"] <= Putpat.getTime() && new Date(clip["start_time"].getTime() + clip["duration"] * 1000) >= Putpat.getTime()) {
            return parseInt(i);
          }
        }
      }
      return -1;
    }

    function findCurrentClipOffset(channel) {
      return findCurrentClipOffsetForChannelId(channel.id);
    }

    function getCurrentClip(channel) {
      var offset = findCurrentClipOffset(channel);
      if (offset >= 0) {
        return channels[channel.id][offset];
      }
      return null;
    }

    function getNextClips(channel, count) {
      var offset = findCurrentClipOffset(channel);
      var clips = [];
      if (offset >= 0) {
        for (var i = 0; i < count; i++) {
          if (channels[channel.id][(offset + i)] != null) {
            clips.push(channels[channel.id][(offset + i)]);
          }
        }
      }
      return clips;
    }

    function getClipsForChannel(channel, callback) {
      fetchClips(channel, function (data) {
        if (data.length > 0) {
          channels[channel.id] = data;
        }
        callback();
      });
    }

    function fixClipStartTimes(clips) {
      var start = Putpat.getTime();
      for (var i in clips) {
        clip = clips[i];
        clip["start_time"] = start;
        start = new Date(start.getTime() + clip["duration"] * 1000);
      }
      return clips;
    }

    self.getCurrentClipForChannel = function (channel, callback) {
      var clip = getCurrentClip(channel);
      if (clip == null) {
        getClipsForChannel(channel, function () {
          try {
            clip = getCurrentClip(channel);
            callback(clip);
          } catch (e) {
            Putpat.debug("error 1");
            Putpat.debug(e.message);
          }
        });
      } else {
        try {
          callback(clip);
        } catch (e) {
         Putpat.debug("error 2");
         Putpat.debug(e.message);
        }
      }
    };

    self.getNextClipsForChannel = function (channel, count, callback) {
      var clips = getNextClips(channel, count);
      if (clips.length == 0) {
        getClipsForChannel(channel, function () {
          try {
            var clips = getNextClips(channel, count);
            callback(clips);
         } catch (e) {
            Putpat.debug("error 3");
            Putpat.debug(e.message);
          }
        });
      } else {
        try {
          callback(clips);
        } catch (e) {
         Putpat.debug("error 4");
         Putpat.debug(e.message);
        }
      }
    };

    self.nextVQClip = function (url) {
      clips = channels[0];

      var offset = findCurrentClipOffsetForChannelId(0);

      if (clips != null && clips.length > offset + 1) {
        clips.splice(0, offset + 1);
        fixClipStartTimes(clips);
        channels[0] = clips;
      } else {
        channels[0] = [];
      }
    };

    self.getClipInfo = function (url) {
      return videoData[url];
    };

    self.clearChannelPlaylist = function (id) {
      channels[id] = null;
    };

    self.loadPlaylist = function(playlistTag, assetId, callback) {
      var params = {}
      params["tag"] = playlistTag;
      if (assetId != null) {
        params["startAssetId"] = assetId;
      }
      params["streamingMethod"] = "http";

      Putpat.debug("lade Clips für playliste!");
      Putpat.callApi("Playlist.clips", params, function(data) {
        var clipList = [];
        if (data != null) {
          for (var i in data) {
            if (data[i]["clip"]) {
              clipList.push(parseClip(data[i]["clip"]));
            }
          }
        }

        channels[0] = clipList;
        currentVQ = null;
        callback();
      });

    };

    self.loadClip = function(assetId, token, callback) {
      Putpat.debug("lade Clips!");
      var params = {}
      params["assetId"] = assetId;

      Putpat.callApi("Veequalizer.presetForAsset", params, function(data) {
        currentVQ = data["veequalizer_preset"];

        var params = {}
        params["token"] = token;
        params["streamingMethod"] = "http";
        Putpat.callApi("Asset.getClipForToken", params, function(data) {
          var clipList = [];
          if (data != null) {
            for (var i in data) {
              if (data[i]["clip"]) {
                clipList.push(parseClip(data[i]["clip"]));
              }
            }
          }
          channels[0] = clipList;
          callback();
        });
      });
    };

    self.loadAssetToken = function(token, callback) {
      Putpat.debug("lade Clips!");

      var params = {}
      params["token"] = token;
      params["streamingMethod"] = "http";
      Putpat.callApi("Asset.getClipForToken", params, function(data) {
        var clipList = [];
        var assetId = null;
        if (data != null) {
          for (var i in data) {
            if (data[i]["clip"]) {
              assetId = data[i]["clip"]["asset"]["id"];
              clipList.push(parseClip(data[i]["clip"]));
            }
          }
        }
        channels[0] = clipList;

        var params = {}
        params["assetId"] = assetId;

        Putpat.callApi("Veequalizer.presetForAsset", params, function(data) {
          currentVQ = data["veequalizer_preset"];
          callback();
        });
      });
    };

    self.resetCurrentVq = function() {
      channels[0] = null;
      currentVQ = null;
    };

    self.currentVqSetting = function(callback) {
      if (currentVQ != null) {
        callback(currentVQ);
      } else {
        Putpat.getCurrentVeequalizer(function(vq) {
          callback(vq);
        });
      }
    };

    self.isCurrentChannelPlayingAds = function() {

      if (ChannelView.currentChannel &&
          typeof ChannelView.currentChannel.disable_advertisement !== 'undefined') {

            if (!!ChannelView.currentChannel.disable_advertisement) {
              Putpat.debug("Channel spielt keine Werbung");
              return false;
            }
          }

      if (ChannelView.lastClip &&
          ChannelView.lastClip.clipInfo &&
          ChannelView.lastClip.clipInfo.clip_container &&
          typeof ChannelView.lastClip.clipInfo.clip_container.disable_advertisement !== 'undefined') {

            var playsAds=  !!!ChannelView.lastClip.clipInfo.clip_container.disable_advertisement;
            Putpat.debug("ClipContainer spielt Werbung: " + playsAds);
            return playsAds;
          }

      // Wenn nichts angegeben ist, spielen wir Werbung
      return true;
    };

    self.getCurrentChannelColor = function(callback) {
      var color = "#00b4ff";

      if (ChannelView.currentChannel && 
          ChannelView.currentChannel.color) {
        color = ChannelView.currentChannel.color;
      }

      if (ChannelView.currentClip &&
          ChannelView.currentClip.clipInfo &&
          ChannelView.currentClip.clipInfo.clip_container &&
          !Utils.isStringEmpty(ChannelView.currentClip.clipInfo.clip_container.color)) {
              color = ChannelView.currentClip.clipInfo.clip_container.color;
      } else if (ChannelView.currentChannel &&
                 ChannelView.currentChannel.id == 0 && 
                 currentVQ && currentVQ.entries) {
        var max_slider = 0;

        for(var i = 0;i<5;i++) {
          if (currentVQ.entries[i]) {
            var value = currentVQ.entries[i].slider_value
            if (value > max_slider) {
              max_slider = value;
              if (!Utils.isStringEmpty(currentVQ.entries[i].color)) {
                color = Utils.convertColor(currentVQ.entries[i].color);
              }
            }
          }
        }

      }
      if (color == null) {
        color = "#00b4ff";
      }
      callback(color);
    }

    return self;
}();
