/**
 * Der globale Keyhandler. Hier werden die 
 * Tastendrücke gefangen und an die registrierten
 * Komponenten weitergeben
 */
var InputHandler = function() {
  var self = {};

  var keyboardHandler = null;
  var keyboardPressHandler = null;
  var keyboardUpHandler = null;

  self.mouseActive = false;

  self.mouseMove = function(event) {
    self.mouseActive = true;
    if (Device.hasMouse && ChannelView.currentView == null && ChannelView.currentChannel != null) {
      ControlBar.show();
    }
  }

  self.mouseClick = function(event) {
    if (Device.hasMouse && ChannelView.currentView == null && ChannelView.currentChannel != null) {
      ControlBar.show();
    }
  }

  /**
   * Das hier ist das Herzstück. Damit
   * das ganze funktioniert muss ein Array mit
   * Keycodes in der Variable KEYS vorhanden sein.
   * Samsung liefert das mit, auf den anderen
   * Geräten gibt es eine keys.js, die diese enthält
   */
  self.keyDown = function(event) {
    var keycode = event.keyCode;

    // Auf Wunsch von Samsung die Untertitel-Taste nicht mehr
    // benutzen, weil da sonst der Artist-Titel angezeigt wird,
    // wollen die aber nicht.
    if (event.keyCode == KEYS.SUBTITLE || event.keyCode == 652) {
      return;
    }

    Putpat.debug("Key DOWN EVENT!");

    if (typeof(event.preventDefault) == "undefined") {
      event.preventDefault = function() { 
      };
    }

    Putpat.debug("KEYCODE : " + keycode);

    try {
      // Return-Knopf immer blockieren
      if (keycode == KEYS.KEY_RETURN || keycode == KEYS.KEY_EXIT) {
        event.preventDefault();
        widgetAPI.blockNavigation(event);
      }

      // Im Samsung ist der Exit-Kopf leider nicht als Key definiert :(
      if (keycode == 147 && Device.name == "Samsung") {
        ChannelView.shutdownApp(false, event);
      } else if (keycode == KEYS.KEY_EXIT) {
        // Exit-Knopf gedrückt
        event.preventDefault();

        if (ErrorView.visible()) {
          // Wenn der Fehlerscreen sichtbar ist, direkt raus
          widgetAPI.sendExitEvent();
        } else {
          // Ansonsten sauber runterfahren
          ChannelView.shutdownApp(true, event);
        }
      } else if (keycode == KEYS.KEY_VOL_UP || keycode == KEYS.KEY_PANEL_VOL_UP) {
          event.preventDefault();
          Audio.volumeUp();
      } else if (keycode == KEYS.KEY_VOL_DOWN || keycode == KEYS.KEY_PANEL_VOL_DOWN) {
          event.preventDefault();
          Audio.volumeDown();
      } else if (keycode == KEYS.KEY_VOL_MUTE || keycode == 15  || keycode == 27 ) {
          event.preventDefault();
          Audio.toggleMute();
      } else if (keycode == KEYS.KEY_PAUSE) {
          event.preventDefault();
          ChannelView.pauseVideo();
      } else if (keycode == KEYS.KEY_PLAY) {
          event.preventDefault();
          ChannelView.playVideo();
      } else if (keyboardHandler != null) {
          // Wenn ein keyboardHandler definiert ist, dann
          // hat ein Screen gerade den Fokus und will die Events
          // haben
          keyboardHandler(event);
      } else {
          // Ansonsten, wenn keine Werbung läuft, die
          // normalen Action Knöpfe
          if (ChannelView.mode != "ADVERT") {
              switch(keycode) {
                  case KEYS.KEY_RIGHT:
                  case KEYS.KEY_FF:
                      event.preventDefault();
                      ChannelView.skipVideo();
                      break;
                  case KEYS.KEY_PANEL_CH_UP:
                  case KEYS.KEY_CH_UP:
                      event.preventDefault();
                      ChannelView.moveChannel(1);
                      break;
                  case KEYS.KEY_PANEL_CH_DOWN:
                  case KEYS.KEY_CH_DOWN:
                      event.preventDefault();
                      ChannelView.moveChannel(-1);
                      break;
                  case KEYS.KEY_RETURN:
                      ChannelView.shutdownApp(false, event);
                      break;
                  case KEYS.KEY_ENTER:
                      if (ChannelView.currentView == null && ChannelView.currentChannel != null) {
                        event.preventDefault();
                        ControlBar.show();
                      }
                      break;
               }
          }
      }
    } catch (e) {
      Putpat.debug("error 5");
      Putpat.debug(e.name);
      Putpat.debug(e.message);
      Putpat.debug("-----!");
    }
  }

  self.keyUp = function(event) {
    if (keyboardUpHandler != null) {
        keyboardUpHandler(event);
    }
  }

  self.keyPress = function(event) {
    if (keyboardPressHandler != null) {
        keyboardPressHandler(event);
    }
  }

  /**
   * Einen neuen Keyhandler registrieren und dann
   * den alten zurückliefern. Man muss, wenn man die
   * Tastatur-Events nicht mehr haben will, den 
   * alten wieder registrieren, sonst gibt's
   * haue
   */
  self.replaceKeyHandlers = function(handlers) {
     var oldHandlers = {
       keyboardHandler: keyboardHandler,
       keyboardUpHandler: keyboardUpHandler,
       keyboardPressHandler: keyboardPressHandler,
     }
     if (handlers == null) {
       handlers = {
         keyboardHandler: function() {}
       };
     }
     keyboardHandler = handlers.keyboardHandler;
     keyboardUpHandler = handlers.keyboardUpHandler;
     keyboardPressHandler = handlers.keyboardPressHandler;
     return oldHandlers;
  }

  return self;
}();
