/**
 * Das Samsung Messaging Interface. Von Außen
 * kann über diese Schnittstelle mit der App kommuniziert
 * werden. Damit kann man nun die App fernsteuern
 */
Messaging = function () {
  var self = {};

  var custom = window.deviceapis.customdevice || {};
  var deviceInstance = [];

  // Listener registrieren
  self.init = function() {
    Putpat.debug("## init messaging");
    custom.registerManagerCallback(onDeviceStatusChange);
    custom.getCustomDevices(onCustomObtained);
  };

  // Device Connection Listener
  onDeviceStatusChange = function(sParam) {
    Putpat.debug("# Device status change recieved #");
    Putpat.debug("# event type is " + sParam.eventType + " #");
    Putpat.debug("# event device name is " + sParam.name + " #");
    Putpat.debug("# event device type is " + sParam.deviceType + " #");

    switch( Number(sParam.eventType) ) {
      case custom.MGR_EVENT_DEV_CONNECT: {
        Putpat.debug("# MGR_EVENT_DEV_CONNECT #");
        break;
      }
      case custom.MGR_EVENT_DEV_DISCONNECT: {
        Putpat.debug("# MGR_EVENT_DEV_DISCONNECT #");
        break;
      }
      default: {
        Putpat.debug("# Unknown event #");
        break;
      }
    }
    custom.getCustomDevices(onCustomObtained);
  };

  onCustomObtained = function(customs){
    Putpat.debug("# onCustomObtained - found " + customs.length + " custom device(s) #");
    for(i=0; i<customs.length; i++) {
      if(customs[i]!=null && customs[i].getType() == custom.DEV_SMART_DEVICE) {
          Putpat.debug("# onCustomObtained - get device instance:" + i);
          deviceInstance[i] = customs[i];
          deviceInstance[i].registerDeviceCallback(onDeviceEvent);
      }
    }
  };

  onDeviceEvent = function(sParam) {
    Putpat.debug("# - First smart device - #");
    switch(Number(sParam.infoType)) {
      case custom.DEV_EVENT_MESSAGE_RECEIVED:
        Putpat.debug("# onDeviceEvent -1- DEV_EVENT_MESSAGE_RECEIVED:" + sParam.data.message1);
        onMessageReceived(sParam.data.message1, sParam.data.message2);
        break;
      case custom.DEV_EVENT_JOINED_GROUP:
        Putpat.debug("# onDeviceEvent -1- DEV_EVENT_JOINED_GROUP #");
        break;
      case custom.DEV_EVENT_LEFT_GROUP:
        Putpat.debug("# onDeviceEvent -1- DEV_EVENT_LEFT_GROUP #");
        break;
      default:
        Putpat.debug("# onDeviceEvent -1- Unknown event #");
        break;
    }
  };

  // Message an den verbundenen Client zurückschicken
  // als Antwort
  sendMessage = function(type, data) {
    for (var i in deviceInstance) {
      var device = deviceInstance[i];
      var message = {'type':type, 'data': data};
      device.sendMessage(JSON.stringify(message));
    }
  };

  // Eine Message wurde empfangen vom verbundenen
  // Client. Diese jetzt auswerten
  onMessageReceived = function(message, context) {
    Putpat.debug("# onMessageReceived:" + message);
    var json = JSON.parse(message);
    if (json.type == "requestVeequalizer") {
      veequalizerRequested();
    } else if (json.type == "requestChannelId") {
      sendMessage("channelId", {channelId:ChannelView.currentChannel["id"]});
    } else if (json.type == "requestCurrentClip") {
      currentClipRequested();
    } else if (json.type == "startVeequalizer") {
      Putpat.debug("setting new vq");
      Putpat.updateVeequalizer(json.data, function() {
        Putpat.getChannels(function(channels) {
           ChannelView.switchToChannel(channels[0], true);
        });
      });
    } else if (json.type == "updateVeequalizer") {
      Putpat.debug("updating vq");
      MessageQueue.sendMessage("updateVeequalizer", json.data);
    } else if (json.type == "playPlaylist") {
      Putpat.debug("playing playlist");
      ChannelView.startPlaylist(json.data.playlist, json.data.asset_id);
    } else if (json.type == "playAsset") {
      Putpat.debug("playing asset");
      ChannelView.startClip(json.data.asset_id, json.data.token);
    } else if (json.type == "playChannel") {
      Putpat.getChannels(function(channels) {
        selectedChannel = null;
        for (var i=0;i<channels.length;i++) {
          if (json.data.channel_id == channels[i].id) {
            selectedChannel = channels[i];
          }
        }
        if (selectedChannel != null) {
          ChannelView.switchToChannel(selectedChannel);
        }
      });
    } else if (json.type == "play") {
      ChannelView.playVideo();
    } else if (json.type == "pause") {
      ChannelView.pauseVideo();
    } else if (json.type == "skip") {
      ChannelView.skipVideo();
    } else if (json.type == "volume_up") {
      Audio.volumeUp();
    } else if (json.type == "volume_down") {
      Audio.volumeDown();
    } else if (json.type == "get_player_id") {
      MessageQueue.sendMessage("player_id", {"player_id": Putpat.getPlayerId()});
    } else if (json.type == "get_user") {
      MessageQueue.sendMessage("user", {"user": Settings.getCurrentUser()});
    } else if (json.type == "user_login") {
      var user = {
        type: "LOGIN",
        login: json.data.username,
        password: json.data.password
      };
      Putpat.trySignup(user, function(success) {
        if (success) {
          Settings.setCurrentUser(user);
          Putpat.getChannels(function(channels) {
            ChannelView.switchToChannel(channels[0], true, true);
          });
        } else {
          Putpat.trySignup(Settings.getCurrentUser(), function(success) {});
        }
      });
    }
  };

  veequalizerRequested = function() {
    Putpat.getCurrentVeequalizer(function(vq){
      Putpat.debug("veequalizer requested!");
      sendMessage("veequalizer", vq);
    });
  };

  currentClipRequested = function() {
    if (ChannelView.currentClip != null) {
      clip = {
        assetId: ChannelView.currentClip.asset_id,
        artist: ChannelView.currentClip.artist,
        title: ChannelView.currentClip.title,
        type: ChannelView.currentClip.asset_type
      };
      sendMessage("currentClip", clip);
    } else {
      sendMessage("currentClip", {empty: true});
    }
  };

  return self;
}();
