var ArtistInfo = function() {
  var self = {};

  var BUTTON_ROTATION = [
    "button_bio",
    "button_tour",
    "button_close"
  ];
  var selected = null;
  var activated = "button_bio";
  var currentClip = null;
  var scrollbarActivated = false;
  var keyboardRepeatTimer = null;

  function resetButtonState() {
    $.each(["button_bio", "button_tour"], function(index, value) { 
      if (activated == value) {
        $("#"+value).css("color", "#ffffff");
      } else {
        $("#"+value).css("color", "#747474");
      }
    });
    $.each(["button_close", "button_scroll_up", "button_scroll_down"], function(index, value) {
      var element = $("#"+value);
      if (element.length > 0) {
        element.attr('src', element.attr('src').replace("_hover.png", ".png"));
      }
    });
    $("#artist_info div#artist_info_bio #scrollbar").hide();
  }

  function selectButton(name) {
    resetButtonState();
    selected = name;
    if (["button_bio", "button_tour"].indexOf(name) != -1) {
      $("#"+name).css("color", "#00b4ff");
    } else {
      $("#"+name).attr('src', $("#"+name).attr('src').replace(".png", "_hover.png"));
    }

    if (["button_scroll_up", "button_scroll_down"].indexOf(name) != -1) {
      $("#artist_info div#artist_info_bio #scrollbar").show();
    }
  }

  function moveHorizontal(dir) {
    var newitem= BUTTON_ROTATION.indexOf(selected) + dir;
    if (newitem < 0) {
      newitem = BUTTON_ROTATION.length - 1;
    } else if (newitem >= BUTTON_ROTATION.length) {
      newitem = 0;
    }

    selectButton(BUTTON_ROTATION[newitem]);
  }

  function moveVertical(dir) {
    if (dir > 0) {
      if ((BUTTON_ROTATION.indexOf(selected) != -1) && scrollbarActivated && (activated=="button_bio") ) {
        selectButton("button_scroll_up");
      } else if (selected == "button_scroll_up") {
        selectButton("button_scroll_down");
      } else{
        resetButtonState();
        ControlBar.focus();
      }
    } else if (dir < 0) {
      if (selected == "button_scroll_down") {
        selectButton("button_scroll_up");
      } else if (selected == "button_scroll_up") {
        selectButton("button_bio");
      }
    }
  }

  function showBio() {
    activated = "button_bio";
    resetButtonState();
    selectButton(activated);
    $("#artist_info_bio").show();
    $("#artist_info_tour").hide();
  }

  function showTour() {
    activated = "button_tour";
    resetButtonState();
    selectButton(activated);
    $("#artist_info_bio").hide();

    if ($("#artist_info_tour").length == 0) {
       $("#artist_info").append("<div id='artist_info_tour'>Suche Tourdaten...</div>");
       Putpat.getTourdates(currentClip["asset"]["artist_id"], function(data) {
          var result = "";

          if (data.length == 0) {
            result += "Leider liegen zu diesem Künstler keine Tourdaten vor.";
          } else {
            result += "<table>";
            jQuery.each(data, function(i, val) {
              if (i < 7) {
                var datumSplit = val.date.date.split("-");
                var datum = new Date(datumSplit[0], datumSplit[1], datumSplit[2]);
                var datumStr = Utils.fillStringLeft(""+datum.getDate(), 2, "0") + "." + Utils.fillStringLeft(""+datum.getMonth(), 2, "0") + "." + datum.getFullYear();
                result += "<tr><td>"+datumStr+"</td><td>"+Utils.replaceWithHTMLEntities(val.date.city)+
                        "</td><td>"+Utils.replaceWithHTMLEntities(val.date.name)+"</td></tr>";
              }
            });
            result += "</table>";
          }

          $("#artist_info_tour").empty().append(result);
      });
    } else {
      $("#artist_info_tour").show();
    }
  }

  function scrollText(dir) {
    var contentEl = $("#artist_info div#artist_info_bio #artist_info_bio_flowtext #artist_info_bio_content");
    var top = 0;

    if (Device.moveTextWithMargin == false) {
      var match = contentEl.css("top").match(/(.*)px/);
  		if (match) {
	    	top = parseInt(match[1], 10);
   		}
    } else {
      var match2 = contentEl.css("margin-top").match(/(.*)px/);
  		if (match2) {
	    	top = parseInt(match2[1], 10);
   		}
     }
    top += dir;

    if ((top*-1) + $("#artist_info_bio_flowtext").height() > contentEl.outerHeight()) {
      top = $("#artist_info_bio_flowtext").height() - contentEl.outerHeight();
    } else if (top > 0) {
      top = 0;
    }

    if (Device.moveTextWithMargin == false) {
      contentEl.css("top", top + "px");
    } else {
      contentEl.css("margin-top", top + "px");
    }

    var height = contentEl.outerHeight();
    var scrollHeight = (100 / (height / ((top * -1) / 100))) * Positions.artist_info.scroll_height;
    $("#artist_info div#artist_info_bio #scrollbar").css("margin-top", scrollHeight+"px");
  }

  function updateScrollbarHeight() {
    $("#artist_info_bio_content").css("width",  Positions.artist_info.text_without_scrolling +"px");
    var height = $("#artist_info_bio_content").outerHeight();
    var targetHeight = $("#artist_info_bio_flowtext").height();

    if (height > targetHeight) {
      $("#artist_info_bio_content").css("width", Positions.artist_info.text_with_scrolling+ "px");
      height = $("#artist_info_bio_content").outerHeight();
      var scrollHeight = (100 / (height / (targetHeight / 100))) * Positions.artist_info.scroll_height;
      $("#artist_info div#artist_info_bio #scrollbar").css("height", scrollHeight + "px");
      $("#artist_info div#artist_info_bio #scrollbar").css("margin-top", "0px");
      $("#artist_info div#artist_info_bio #button_scroll_up").show();
      $("#artist_info div#artist_info_bio #button_scroll_down").show();
      $("#artist_info div#artist_info_bio #scrollbar").hide();
      scrollbarActivated = true;
    } else {
      $("#artist_info div#artist_info_bio #button_scroll_up").hide();
      $("#artist_info div#artist_info_bio #button_scroll_down").hide();
      $("#artist_info div#artist_info_bio #scrollbar").hide();
      scrollbarActivated = false;
    }
  }

  function executeButton() {
    var repeatKey = false;
    if (selected == "button_close") {
      self.close();
    } else if (selected == "button_bio") {
       Putpat.trackIvwUi();
       showBio();
    } else if (selected == "button_tour") {
       Putpat.trackIvwUi();
       showTour();
    } else if (selected == "button_scroll_up") {
      scrollText(30);
      repeatKey = true;
    } else if (selected == "button_scroll_down") {
      scrollText(-30);
      repeatKey = true;
    }
    return repeatKey;
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     var repeatKey = false;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             moveHorizontal(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_RIGHT:
             moveHorizontal(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_UP:
             moveVertical(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             moveVertical(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_ENTER:
             repeatKey = executeButton();
             event.preventDefault();
             break;
         case KEYS.KEY_RETURN:
             self.close();
             event.preventDefault();
             break;
         default:
             break;
     }

     if (repeatKey && ($("#artist_info").length > 0)) {
        keyboardRepeatTimer = setTimeout(function() {
          keyboardHandler(event);
        }, 200);
     }
  }

  function keyboardUpHandler(event) {
    clearTimeout(keyboardRepeatTimer);
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $("#"+button).mouseover(function() {
         ControlBar.unfocus();
         selectButton(this.id);
      });
      $("#"+button).click(function() {
        ControlBar.unfocus();
        if (selected != this.id) {
          selectButton(this.id);
        }
        executeButton(this.id);
      });
    }
  }

  self.show = function(all_graphics) {
    $("#artist_info").remove();
    if (all_graphics) {
      $("#putpat_view").append(
        "<div id='artist_info'>" +
          "<img class='background' src='" + Positions.size + "images/artist_info/bg_artistinfo.jpg'/>" +
        "</div>");
    }

    Tracking.trackAction("impression", "artistinfo", ChannelView.currentClip["artist_id"]);

    Putpat.getAssetInfo(ChannelView.currentClip["asset_id"], function(result) {
      currentClip = result;
      selected = "button_bio";

      $("#artist_info").append(
        "<img id='button_close' class='artist_button' src='"+Positions.size+"images/bt_close.png'/>" + 
        "<div class='artist_label'>KÜNSTLER</div>" +
        "<div class='artist_name'>"+Utils.replaceWithHTMLEntities(currentClip["asset"]["display_artist_title"])+"</div>" +
        "<div id='button_bio' class='artist_button'>BIO</div>" +
        "<div id='button_tour' class='artist_button'>TOURDATEN</div>" +
        "<div id='artist_similar_label'>Ähnliche Künstler</div>");

      wireMouseEvents("button_bio");
      wireMouseEvents("button_tour");
      wireMouseEvents("button_close");

      $("#artist_info").append("<img class='posterframe' src='http://files.putpat.tv/artwork/posterframes/" + Utils.getPathFor(currentClip["asset"]["video_file_id"], "v")+"_posterframe_large.jpg'/>");

      var albumImg = document.createElement('img');
      albumImg.onLoad = albumImg.onload = function() {
          $("#artist_info").append("<img class='album' src='http://files.putpat.tv/artwork/album_covers/" + Utils.getPathFor(currentClip["asset"]["album"]["id"], "")+"_front.jpg'/>");
          albumImg = null;
      };
      albumImg.src = "http://files.putpat.tv/artwork/album_covers/" + Utils.getPathFor(currentClip["asset"]["album"]["id"], "")+"_front.jpg";

      var artistInfo = document.createElement('img');
      artistInfo.onload = function() {
          var copyright = currentClip["asset"]["artist"]["poster_picture_credit"];
          if (copyright == null) {
             copyright = "";
          }

          $("#artist_info").append("<div class='artist'><img src='http://files.putpat.tv/artwork/artist_pictures/" + Utils.getPathFor(currentClip["asset"]["artist"]["id"], "")+
                                   "_01.jpg'/><br/>" + Utils.replaceWithHTMLEntities(copyright)+"</div>");
          artistInfo = null;
      };
      artistInfo.src = "http://files.putpat.tv/artwork/artist_pictures/" + Utils.getPathFor(currentClip["asset"]["artist"]["id"], "")+"_01.jpg";

      if (currentClip["asset"]["album"]["id"] != null) {
        $("#artist_info").append("<div id='artist_info_album_title_headline'>Album</div>");
        $("#artist_info").append("<div id='artist_info_album_title'>" + Utils.replaceWithHTMLEntities(currentClip["asset"]["album"]["title"]) + "<br/>" + currentClip["asset"]["album"]["release_year"]+ "</div>");
      }

      ChannelView.hideChannelLogo();

      var bio = "";
      var wiki = "";

      if (currentClip["asset"]["artist"]) {
        bio = Utils.replaceWithHTMLEntities(currentClip["asset"]["artist"]["artist_bio"]);
        wiki = currentClip["asset"]["artist"]["wiki_link"];
      }

      if (Utils.isStringEmpty(bio)) {
        bio = "Hey!\nFür diesen Künstler existiert leider noch keine Biografie.\nSchade. Du kennst dich mit diesem Künstler aus? Super!\nDann schreib' die Biografie doch einfach selber! Schick' uns deinen Text mit dem Betreff -KÜNSTLERNAME-/BIO und deinem vollen Namen an supporter@putpat.tv .";
      } else if (!Utils.isStringEmpty(wiki)) {
        bio += "<br/><br/><p>Dieser Text basiert auf dem Artikel " + wiki + " aus Wikipedia.</p>";
      }

      if (bio.length > 0){
          $("#artist_info").append("<div id='artist_info_bio'>" + 
                  "<div id='artist_info_bio_flowtext'><div id='artist_info_bio_content'>'"+bio+"</div></div>" +
                  "<img id='button_scroll_up' class='news_button' src='"+Positions.size + "images/bt_up.png'/>" +
                  "<div id='scrollbar'></div>" +
                  "<img id='button_scroll_down' class='news_button' src='"+Positions.size + "images/bt_down.png'/>" +
           "</div>");
          setTimeout(function () {
            updateScrollbarHeight();
            if ($("#artist_info div#artist_info_bio #button_scroll_down").is(":visible")) {
              selectButton("button_scroll_down");
            }
          }, 250);
          wireMouseEvents("button_scroll_up");
          wireMouseEvents("button_scroll_down");
      }

      var similar = "";

      if (currentClip["asset"]["artist"] != null && currentClip["asset"]["artist"]["similar_artist_infos"] != null) {
         for(i = 0; i < 5 && i < currentClip["asset"]["artist"]["similar_artist_infos"].length; i++) {
            similar += Utils.replaceWithHTMLEntities(currentClip["asset"]["artist"]["similar_artist_infos"][i]["title"]) + "<br/>";
         }
      }

      if (similar.length == 0) {
        similar = "...";
      }

      $("#artist_info").append("<div id='artist_info_similar'>"+similar+"</div>");
      self.focus();
    });
    MessageQueue.sendMessage("disable_bauchbinde");
  };

  self.hide = function() {
    $("#artist_info").hide();
    $("#artist_info img").remove();
    MessageQueue.sendMessage("enable_bauchbinde");
    ChannelView.showChannelLogo();
  };

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
      keyboardUpHandler: keyboardUpHandler
    });
    selectButton(selected);
  };

  self.unfocus = function() {
    resetButtonState();
  };

  self.close = function() {
    self.hide();
    ControlBar.currentScreenClosed();
  };

  return self;
}();
