var ChannelSelect = function() {
  var self = {};
  var currentChannel = null;
  var channelArray = null;

  var MAX_CHANNELS = 9;
  var startChannel = 1;
  var selectedChannel = 0;
  var selectedButton = null;
  var visible = false;

  function resetButtonState() {
    var removedEl = $("#channel_select_highlight").remove();
    $("#button_close").attr('src', $("#button_close").attr('src').replace("_hover.png", ".png"));
    selectedChannel = null;
    selectedButton = null;
  }

  function selectButton(button) {
    resetButtonState();
    selectedButton = button;
    $("#button_close").attr('src', $("#button_close").attr('src').replace(".png", "_hover.png"));
  }

  function executeButton() {
    if (selectedButton == "button_close") {
      self.close();
    } else {
      self.close();
      ControlBar.hide();
      ChannelView.switchToChannel(channelArray[selectedChannel]);
    }
  }

  function moveSelection(dir) {
    var oldButton = selectedButton;
    var oldChannel = selectedChannel;
    resetButtonState();
    if (oldButton == "button_close") {
      if (dir > 0) {
        selectedChannel = 0;
        startChannel = 1;
      } else {
        selectedChannel = channelArray.length - 1;
      }
      updateChannelIcons();
    } else {
      if ((dir < 0 && oldChannel == 0) ||
          (dir > 0 && oldChannel == channelArray.length - 1))  {
        selectButton("button_close");
      } else {
        selectedChannel = oldChannel + dir;
        updateChannelIcons();
      }
    }

    if (selectedChannel != null) {
      updateChannelDetails(channelArray[selectedChannel]);
    }
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             moveSelection(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_RIGHT:
             moveSelection(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_ENTER:
             executeButton();
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             resetButtonState();
             ControlBar.focus();
             event.preventDefault();
             break
         case KEYS.KEY_RETURN:
             self.close();
             event.preventDefault();
             break;
     }
  }

  function updateClipDetails(clip) {
    if (!visible) {
      return;
    }

    var clipTitle = "";

    if (!Utils.isStringEmpty(clip.artist)) {
     clipTitle = "<span class='artist'>"+Utils.replaceWithHTMLEntities(clip.artist)+"</span> - ";
    }
    clipTitle += Utils.replaceWithHTMLEntities(clip.title);

    $("#channel_select #channel_info #channel_details #channel_now_running").empty()
          .append(clipTitle);
    $("#channel_select #channel_info #channel_details #channel_now_running_posterframe").empty()
          .append("<img class='posterframe' src='http://files.putpat.tv/artwork/posterframes/" + 
                Utils.getPathFor(clip.video_file_id, "v")+"_posterframe_small.jpg'/>");
  }

  function updateChannelDetails(channel) {
    if (currentChannel != null && currentChannel.id == channel.id) {
      return;
    }

    currentChannel = channel;

    var element = $("#channel_select #channel_info #channel_details");
    element.empty();
    element.append(
      "<img id='channel_details_logo' src='http://files.putpat.tv/artwork/channelgraphics/"+channel.id+"/channellogo_100.png'/>" +
      "<div id='channel_message'>"+Utils.replaceWithHTMLEntities(channel.channel_message)+"</div>" + 
      "<div id='channel_desc'>"+Utils.replaceWithHTMLEntities(channel.channel_info).replace(/\n/g, "<br/>")+"</div>" +
      "<div id='channel_now_running_title' style='color:"+channel.color+"'>Gerade läuft:</div>" + 
      "<div id='channel_now_running'></div><div id='channel_now_running_posterframe'></div>"
    );

    if (channel.id == 0) {
      element.append("<div id='channel_name'>Dein Kanal</div>");
    } else {
      element.append("<div id='channel_name'>Kanal: <span style='color:"+channel.color+"'>"+Utils.replaceWithHTMLEntities(channel.title)+"</span></div>");
    }

    if (channel.id == ChannelView.currentChannel.id && ChannelView.currentClip != null) {
      updateClipDetails(ChannelView.currentClip);
    } else {
      Channels.getCurrentClipForChannel(channel, function(clip) {
        if (channel.id == currentChannel.id && clip != null) {
          updateClipDetails(clip);
        }
      });
    }
  }

  function updateChannelIcons() {
    var element = $("#channel_select #channel_bar #channel_icons");
    element.empty();

    element.append("<img class='veequalizer channel' src='http://files.putpat.tv/artwork/channelgraphics/0/channellogo_100.png' data-channel='0'/>");

    if (selectedChannel > startChannel + MAX_CHANNELS - 2) {
      startChannel = selectedChannel - MAX_CHANNELS + 1;
    } else if (selectedChannel != 0 && selectedChannel < startChannel) {
      startChannel = selectedChannel;
    }

    if (selectedChannel != null) {
      element.append("<img id='channel_select_highlight' "+
                     "style='left:" + Positions.channelbar.select_position[selectedChannel - (startChannel - 1)]+"px' " +
                     "src='" + Positions.size + "images/channels/bt_channel_hover.png' />");
    }

    var channelIcons = "<div id='otherchannels'>";
    for (var i = startChannel; i < (startChannel + MAX_CHANNELS);i++) {
      var channel = channelArray[i];
      if (channel != null) {
        channelIcons += "<img class='channel' src='http://files.putpat.tv/artwork/channelgraphics/"+channel.id+"/channellogo_invert_100.png' data-channel='"+i+"'/>";
      }
    }
    channelIcons += "</div>"
    element.append(channelIcons);

    if (Device.hasMouse) {
      $(".channel").click(function() {
        ControlBar.unfocus();
        selectedButton = null;
        selectedChannel = this.getAttribute("data-channel");
        executeButton();
      });
      $(".channel").mouseover(function() {
         if (this.getAttribute("data-channel") != selectedChannel) {
           ControlBar.unfocus();
           selectedButton = null;
           selectedChannel = this.getAttribute("data-channel");
           updateChannelIcons();
           updateChannelDetails(channelArray[selectedChannel]);
         }
      });
    }
  }

  self.show = function() {
    Tracking.trackAction("impression", "channel_selection");

    ChannelView.hideChannelLogo();
    $("#channel_select").remove();
    $("#putpat_view").append(
      "<div id='channel_select'>"+
        "<div id='channel_info'>" +
           "<img class='background' src='" + Positions.size + "images/channels/bg_channelinfo.png'/>" +
           "<div id='channel_details'></div>"+
        "</div>"+
        "<div id='channel_bar'>"+
           "<img class='background' src='" + Positions.size + "images/channels/bg_channels.jpg'/>" +
           "<img id='button_close' class='artist_button' src='"+Positions.size+"images/bt_close.png'/>" +
           "<div id='channel_icons'></div>" +
        "</div>"+
      "</div>"
    );
    currentChannel = null;
    visible = true;
    Putpat.getChannels(function(channels) {
      channelArray = channels;
      $.each(channelArray, function(i, channel) {
        if (channel.id == ChannelView.currentChannel.id) {
          selectedChannel = i;
          updateChannelDetails(ChannelView.currentChannel);
          currentChannel = channel;
          self.focus();
        }
      });

    });

    MessageQueue.sendMessage("disable_bauchbinde");
    $("#button_close").mouseover(function() {
       ControlBar.unfocus();
       selectButton(this.id)
    });
    $("#button_close").click(function() {
       ControlBar.unfocus();
       selectButton("button_close");
       executeButton();
    });
  }

  self.hide = function() {
    visible = false;
    MessageQueue.sendMessage("enable_bauchbinde");
    $("#channel_select").hide();
    $("#channel_select img").remove();
    ChannelView.showChannelLogo();
  }

  self.close = function() {
    self.hide();
    ControlBar.currentScreenClosed();
  }

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    if (selectedChannel == null) {
      $.each(channelArray, function(i, channel) {
        if (channel.id == currentChannel.id) {
          selectedChannel = i;
        }
      });
    }
    updateChannelIcons();
  }

  self.unfocus = function() {
    resetButtonState();
  }

  return self;
}();
