var ClipsPanel = function() {
  var self = {};

  var BUTTON_ROTATION = [
    "button_play",
    "button_similar",
    "button_love",
    "button_hate",
    "button_new",
    "button_close"
  ];

  var fetchedClipsTime = [];
  var newClips = null;
  var currentNewClip = 0;
  var clipRotationTimer = null;
  var similarEnabled = true;
  var selected = "button_similar";
  var clipsData = {};
  var currentGrid = null;
  var gridStartOffset = 0;

  function getGridPositions(button) {
    var gridMatch = button.match(/grid_(\d*)_(\d*)/);
    if (gridMatch) {
      var x = parseInt(gridMatch[1]) - 1;
      var y = parseInt(gridMatch[2]) - 1;
      return {x: x, y: y};
    }
    return null;
  }

  function resetButtonState() {
    $(".clips_button").each(function(i, el) {
      var name = $(el).attr("id").match(/button_(.*)/)[1];
      if (["close"].indexOf(name) != -1) {
        $(el).attr("src", Positions.size+"images/bt_"+name+".png");
      } else  if (["play", "up", "down"].indexOf(name) != -1) {
        $(el).attr("src", Positions.size+"images/clips/bt_"+name+".png");
      } else if (name != currentGrid) {
        $(el).attr("src", Positions.size+"images/clips/bt_"+name+"_inactive.png");
      } else {
        $(el).attr("src", Positions.size+"images/clips/bt_"+name+"_active.png");
      }
    });
    $("#select_grid").hide();
  }

  function selectButton(button) {
    resetButtonState();
    selected = button;
    var buttonMatch = button.match(/button_(.*)/);
    var gridMatch = getGridPositions(button);
    
    if (buttonMatch) {
      var gfxName = buttonMatch[1];
      if (selected != "button_close") {
        $("#clips_panel #"+button).attr("src", Positions.size+"images/clips/bt_"+gfxName+"_hover.png");
      } else {
        $("#clips_panel #"+button).attr("src", Positions.size+"images/bt_"+gfxName+"_hover.png");
      }

      if (selected == "button_up" || selected == "button_down") {
      } else if (selected == "button_play") {
        resetTimer();
      }
    } else if (gridMatch != null) {
      var element = $("#clips_panel #select_grid");
      element.css("left", Positions.clips.selection_x[gridMatch.x] +"px");
      element.css("top",  Positions.clips.selection_y[gridMatch.y] +"px");
      element.show();
    }
  }

  function executeButton() {
    Putpat.debug(selected);
    if (selected == "button_close") {
      self.close();
    } else if (selected == "button_similar") {
      Putpat.trackIvwUi();
      showSimilarClips();
    } else if (selected == "button_love") {
      Putpat.trackIvwUi();
      showClips("love", "top_rated");
    } else if (selected == "button_hate") {
      Putpat.trackIvwUi();
      showClips("hate", "most_hated");
    } else if (selected == "button_new") {
      Putpat.trackIvwUi();
      showClips("new", "new");
    } else if (selected == "button_play") {
      Putpat.debug("button play pressed!");
      self.close();
      ControlBar.hide();
      Tracking.trackAction("user_action", "clip_banner_click", newClips[currentNewClip].video_release.asset_id);
      ChannelView.startPlaylist(
        newClips[currentNewClip].video_release.associated_playlist_name,
        newClips[currentNewClip].video_release.asset_id
      );
    } else if (selected == "button_down") {
      gridScroll(1);
    } else if (selected == "button_up") {
      gridScroll(-1);
    } else {
       var gridMatch = getGridPositions(selected);
       if (gridMatch != null) {
          var selClip = (gridMatch.x + gridMatch.y * 3) + gridStartOffset;
          self.close();
          ControlBar.hide();
          Tracking.trackAction("user_action", "clip_search_click", clipsData[currentGrid][selClip].asset.id);
          ChannelView.startClip(clipsData[currentGrid][selClip].asset.id,
                                clipsData[currentGrid][selClip].asset.token);
       }
    }
  }

  function moveSelectionHorizontal(dir) {
    if (selected == "button_play" && dir == 1) {
      selectButton("grid_1_3");
    } else if ((selected == "button_up") && (dir == -1)) {
      selectButton("grid_3_1");
    } else if ((selected == "button_down") && (dir == -1)) {
      selectButton("grid_3_3");
    } else if (BUTTON_ROTATION.indexOf(selected) != -1) {
      var newitem= BUTTON_ROTATION.indexOf(selected) + dir;
      if (newitem < 0) {
        newitem = BUTTON_ROTATION.length - 1;
      } else if (newitem >= BUTTON_ROTATION.length) {
        newitem = 0;
      }
      if (BUTTON_ROTATION[newitem] == "button_similar" && !similarEnabled) {
        newitem += dir;
      }

      selectButton(BUTTON_ROTATION[newitem]);
    } else {
      var gridMatch = getGridPositions(selected);
      if (gridMatch != null) {
        var newX = gridMatch.x + dir;
        var newY = gridMatch.y;
        if (newX < 0) {
          selectButton("button_play");
        } else if (newX >= 3) {
          if (newY == 0) {
            selectButton("button_up");
          } else {
            selectButton("button_down");
          }
        } else {
          selectButton("grid_"+(newX+1)+"_"+(gridMatch.y+1));
        }
      }
    }
  }

  function moveSelectionVertical(dir) {
    var gridMatch = getGridPositions(selected);
    if (gridMatch != null) {
        var newY = gridMatch.y + dir;
        if (newY < 0) {
          if (gridMatch.x == 0) {
            if (similarEnabled) {
              selectButton("button_similar");
            } else {
              selectButton("button_love");
            }
          } else if (gridMatch.x == 1) {
            selectButton("button_love");
          } else {
            selectButton("button_new");
          }
        } else if (newY >= 3) {
          selectButton("button_down");
        } else {
          selectButton("grid_"+(gridMatch.x+1)+"_"+(newY+1));
        }
    } else if (dir == 1) {
      if (selected == "button_similar") {
        selectButton("grid_1_1");
      } else if (selected == "button_love" || selected == "button_hate") {
        selectButton("grid_2_1");
      } else if (selected == "button_close") {
        selectButton("button_up");
      } else if (selected == "button_up") {
        selectButton("button_down");
      } else if (selected == "button_new") {
        selectButton("grid_3_1");
      } else if (selected == "button_play" || selected == "button_down") {
        resetButtonState();
        ControlBar.focus();
      }
    } else if (dir == -1) {
      if (selected == "grid_1_1") {
        selectButton("button_similar");
      } else if (selected == "grid_2_1") {
        selectButton("button_love");
      } else if (selected == "grid_3_1") {
        selectButton("button_new");
      } else if (selected == "button_down") {
        selectButton("button_up");
      } else if (selected == "button_up") {
        selectButton("button_close");
      }
    }
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             moveSelectionHorizontal(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_RIGHT:
             moveSelectionHorizontal(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_UP:
             moveSelectionVertical(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             moveSelectionVertical(+1);
             event.preventDefault();
             break;
          case KEYS.KEY_ENTER:
             executeButton();
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             resetButtonState();
             ControlBar.focus();
             event.preventDefault();
             break
         case KEYS.KEY_RETURN:
             self.close();
             event.preventDefault();
             break;
     }
  }

  function updateNewClips() {
    $("#new_clip").attr("src", newClips[currentNewClip].video_release.image);

    $("#clips_panel #new_clip_label").empty().append(newClips[currentNewClip].video_release.type);
    $("#new_clip_name").empty().append("<span class='headline'>" +
                                    Utils.replaceWithHTMLEntities(newClips[currentNewClip].video_release.headline) +
                                  "</span><br/>" + 
                                  Utils.replaceWithHTMLEntities(newClips[currentNewClip].video_release.sub_headline));
    $("#clips_panel .dot").attr("src", Positions.size+"images/clips/dot_inactive.png");
    $("#clips_panel #dot_" + (currentNewClip + 1)).attr("src", Positions.size+"images/clips/dot_active.png");
  }

  function showNewClipBanner(callback) {
    // Wenn noch nie geladen oder vor mehr als 2 Stunden geladen, neu holen
    if (fetchedClipsTime["new_banner"] == null || fetchedClipsTime["new_banner"].getTime() + 2 * 60 * 60 * 1000 > new Date().getTime()) {
      Putpat.getNewVideoReleases(function(data) {
        fetchedClipsTime["new_banner"] = new Date();
        newClips = data;
        updateNewClips();
        if (callback != null) {
          callback();
        }
      });
    } else {
      newClips = data;
      updateNewClips();
      if (callback != null) {
         callback();
      }
    }
  }

  function rotateNewClipImage() {
    currentNewClip++;
    if (currentNewClip >= newClips.length) {
      currentNewClip = 0;
    }
    updateNewClips();
  }

  function updateGrid() {
    for (var i = 0; i < 9; i++) {
      var clip = clipsData[currentGrid][i + gridStartOffset];
      if (clip != null && clip != undefined) {
        $("#clips_panel #grid_"+i+"_img").attr("src",
               "http://files.putpat.tv/artwork/posterframes/" +
               Utils.getPathFor(clip.asset.video_file.id, "v") +
               "_posterframe_putpat_small.jpg"
        );
        $("#clips_panel #grid_"+i+"_artist").empty().append(
          Utils.replaceWithHTMLEntities(clip.asset.display_artist_title)
        );
        $("#clips_panel #grid_"+i+"_title").empty().append(
          Utils.replaceWithHTMLEntities(clip.asset.title)
        );
        $("#clips_panel #grid_"+i).show();
       } else {
        $("#clips_panel #grid_"+i).hide();
       }
    }

    var height = Positions.clips.scroll_height / 6 * 3;
    var top = (Positions.clips.scroll_height / 6) * (gridStartOffset / 3);

    $("#clips_panel #scrollbar").css("margin-top", top + "px");
    $("#clips_panel #scrollbar").css("height", height + "px");
  }

  function setCurrentGrid(grid) {
    currentGrid = grid;
    gridStartOffset = 0;
    updateGrid();
    selectButton(selected);
  }

  function gridScroll(dir) {
    var oldStart = gridStartOffset;
    gridStartOffset += 3 * dir;

    if (gridStartOffset < 0) {
      gridStartOffset = 0;
    } else if (gridStartOffset + 9 >= clipsData[currentGrid].length) {
      gridStartOffset = clipsData[currentGrid].length - 9;
    }

    if (oldStart != gridStartOffset) {
      updateGrid();
    }
  }

  function showSimilarClips() {
    if (clipsData.similar) {
      setCurrentGrid("similar");
    } else {
      Putpat.callApi("Asset.similarAssets",
                    {"assetId": ChannelView.currentClip.asset_id, "limit" : 18},
          function(data) {
            clipsData.similar = data;
            setCurrentGrid("similar");
          }
      );
    }
  }

  function showClips(button, type) {
    if (clipsData[button] && (fetchedClipsTime[type] != null && (fetchedClipsTime[type].getTime() + 2 * 60 * 60 * 1000 > new Date().getTime()))) {
      setCurrentGrid(button);
    } else {
       Putpat.callApi("Asset.collection",
                    {"type": type, "genre": "All", "limit" : 18, "timespan" : "week"},
          function(data) {
            fetchedClipsTime[type] = new Date();
            clipsData[button] = data;
            setCurrentGrid(button);
          }
      );
    }
  }

  function resetTimer() {
    if (clipRotationTimer != null) {
      clearInterval(clipRotationTimer);
    }
    clipRotationTimer = setInterval(rotateNewClipImage, 8000);
  }

  function getNameForId(id) {
    var number = id.match(/grid_(.*)/);
    if (number) {
      var num = number[1];
      var cell = (num % 3) + 1;
      var row = (Math.floor(num / 3)) + 1;
      return "grid_"+cell+"_"+row;
    }
    return id;
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $(button).mouseover(function() {
         ControlBar.unfocus();
         var id = getNameForId(this.id);
         Putpat.debug("ID : " + id);
         if (selected != id && id != "select_grid") {
           selectButton(id);
         }
      });
      $(button).click(function() {
        ControlBar.unfocus();
        var id = getNameForId(this.id);
        Putpat.debug("Click " + id);
        if (selected != id && id != "select_grid") {
          selectButton(id)
        };
        executeButton(id);
      });
    }
  }

  self.show = function() {
    Tracking.trackAction("impression", "clips");

    ChannelView.hideChannelLogo();
    $("#clips_panel").remove();

    var gridHTML = "";

    for (var i = 0;i<9;i++) {
      gridHTML += "<div class='grid_item' id='grid_"+i+"'>"+
          "<img class='picture' id='grid_"+i+"_img' />"+
          "<div class='artist' id='grid_"+i+"_artist'></div>"+
          "<div class='title' id='grid_"+i+"_title'></div>"+
        "</div>";
    }

    $("#putpat_view").append(
      "<div id='clips_panel'>" +
        "<img class='background' src='" + Positions.size + "images/clips/bg_clips.jpg'/>" +
        "<img id='button_close' class='clips_button' src='"+Positions.size+"images/bt_close.png'/>" +
        "<img id='button_up' class='clips_button' src='"+Positions.size+"images/clips/bt_up.png'/>" +
        "<img id='button_down' class='clips_button' src='"+Positions.size+"images/clips/bt_down.png'/>" +
        "<div id='scrollbar'></div>"+
        "<img id='new_clip' />" +
        "<div id='new_clip_label'>Clip-Premiere</div>" +
        "<div id='new_clip_name'></div>" +
        "<img id='button_play' class='clips_button' src='"+Positions.size+"images/clips/bt_play.png'/>" +
        "<img id='dot_1' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='dot_2' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='dot_3' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='dot_4' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='dot_5' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='dot_6' class='dot' src='"+Positions.size+"images/clips/dot_inactive.png'/>" + 
        "<img id='button_similar' class='clips_button grid_button' src='"+Positions.size+"images/clips/bt_similar_inactive.png'/>" +
        "<img id='button_love' class='clips_button grid_button' src='"+Positions.size+"images/clips/bt_love_inactive.png'/>" +
        "<img id='button_hate' class='clips_button grid_button' src='"+Positions.size+"images/clips/bt_hate_inactive.png'/>" +
        "<img id='button_new' class='clips_button grid_button' src='"+Positions.size+"images/clips/bt_new_inactive.png'/>" +
        gridHTML +
        "<img id='select_grid' src='"+Positions.size+"images/clips/clip_item_hover.png'/> "+
      "</div>"
    );

    showNewClipBanner(function() {
      resetTimer();
    });

    var clip = ChannelView.currentClip;
    currentGrid = null;

    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });

    if (clip && clip.asset_type == "MusicVideo") {
      similarEnabled = true;
      clipsData.similar = null;
      showSimilarClips();
      $("#button_similar").css("opacity", "1");
      selectButton("button_similar");
    } else {
      similarEnabled = false;
      showClips("love", "top_rated");
      $("#button_similar").css("opacity", "0.3");
      selectButton("button_love");
    }
    MessageQueue.sendMessage("disable_bauchbinde");

    wireMouseEvents(".grid_item");
    wireMouseEvents(".grid_button");
    wireMouseEvents(".clips_button");
    wireMouseEvents("#select_grid");
  }

  self.hide = function() {
    MessageQueue.sendMessage("enable_bauchbinde");
    $("#clips_panel").hide();
    $("#clips_panel img").remove();
    ChannelView.showChannelLogo();
    clearInterval(clipRotationTimer);
  }

  self.close = function() {
    self.hide();
    ControlBar.currentScreenClosed();
  }

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    selectButton("grid_1_3");
  }

  self.unfocus = function() {
    resetButtonState();
  }

  return self;
}();
