var NewsPanel = function() {
  var self = {};

  var MAX_NEWS = 5;

  var selected = null;
  var startOffset = 0;
  var visible = false;
  var newsAllLoaded = false;
  var loadingNews = false;
  var newsItems = [];
  var newsCache = [];
  var selectedNewsBox = 0;
  var selectedNews = 0;
  var playlistActivated = false;
  var scrollbarActivated = false;
  var keyboardRepeatTimer = null;

  function resetButtonState() {
    $("#news_panel #news_highlight").hide();

    $(".news_button").each(function(i) {
      $(this).attr('src', $(this).attr('src').replace("_hover.png", ".png"));
    });
  }

  function updateScrollbarVisibility() {
    if (selected == "button_scroll_up" || selected == "button_scroll_down") {
      $("#news_scrollbar").show();
    } else {
      $("#news_scrollbar").hide();
    }
  }

  function selectButton(button) {
    resetButtonState();
    selected = button;

    var regResult = selected.match(/news_box_(.*)/)

    if (regResult) {
      $("#news_panel #news_highlight").show();
      $("#news_panel #news_highlight").css("left", Positions.news.select_position[regResult[1]]);
      selectedNewsBox = regResult[1];
    } else {
      $("#"+selected).attr('src', $("#"+selected).attr('src').replace(".png", "_hover.png"));
    }
    updateScrollbarVisibility();
  }

  function updateNewsDetails(news) {
    item = news.news;
    Tracking.trackAction("impression", "news", item.id);

    $("#news_headline").empty().append(Utils.replaceWithHTMLEntities(item.headline) +
                                  ": <span class='subhead'>" +
                                  Utils.replaceWithHTMLEntities(item["sub_headline"]) +
                                  "</span>");

    $("#news_content").empty().append(item.content);

    if (Device.moveTextWithMargin == false) {
      $("#news_content").css("top", "0px");
    } else {
      $("#news_content").css("margin-top", "0px");
    }

    $("#news_photo_copyright").empty();
    if (!Utils.isStringEmpty(item.image_credits)) {
      $("#news_photo_copyright").append("Foto: " + item.image_credits);
    }

    var str = Utils.fillStringLeft(""+item.id, 7, 0);
    var imageName = "";
    if (!Utils.isStringEmpty(item.image_full_file_name)) {
      imageName = item.image_full_file_name + "_full.jpg";
    } else {
      imageName = item.image_mid_file_name + "_mid.jpg";
    }

    var imagePath = "http://files.putpat.tv/artwork/news_images/" +
                  str.substring(0,5) + "/" + str + "/"+imageName;

    var imageHandle = document.createElement('img');;
    $("#news_photo_img").css("height", "200px");

    var imageForNews = selectedNews;
    imageHandle.onLoad = imageHandle.onload = function() {
      var height = 355 * (imageHandle.height / imageHandle.width);
      if (imageForNews == selectedNews) {
        $("#news_photo_img").css("height", Math.round(height)+"px");
      }
      imageHandle = null;
    };
    imageHandle.src = imagePath;

    $("#news_photo_img").attr("src", imagePath);

    if (!Utils.isStringEmpty(item.associated_playlist_name)) {
      $("#news_panel #button_play").show();
      playlistActivated = true;
    } else {
      $("#news_panel #button_play").hide();
      playlistActivated = false;
    }

    var height = $("#news_content").height();

    if (height > Positions.news.text_height) {
      var scrollHeight = (100 / (height / (Positions.news.text_height / 100))) * Positions.news.scroll_height;
      $("#news_panel #news_scrollbar").css("height", scrollHeight + "px");
      $("#news_panel #news_scrollbar").css("margin-top", "0px");
      $("#news_panel #button_scroll_up").show();
      $("#news_panel #button_scroll_down").show();
      scrollbarActivated = true;
    } else {
      $("#news_panel #button_scroll_up").hide();
      $("#news_panel #button_scroll_down").hide();
      scrollbarActivated = false;
    }
    updateScrollbarVisibility();
  }

  function updateSelectedBorder() {
    if (selectedNews >= startOffset && selectedNews < startOffset + MAX_NEWS) {
      var date = newsItems[selectedNews].news.publishable_from.match(/(\d*)-(\d*)-(\d*)T.*/)
      $("#news_selected_date").empty().append(date[3]+"."+date[2]+"."+date[1]);
      $("#news_selected").css("left", Positions.news.select_position[selectedNews - startOffset]);
      $("#news_selected").show();
    } else {
      $("#news_selected").hide();
    }
  }

  function getDetailedNews(newsId) {
    var result = null;
    $.each(newsCache, function(i, news){
      if (news.news.id == newsId) {
        result =  news;
        return;
      }
    });
    return result;
  }

  function showNews(pos, newsId) {
    selectedNews = startOffset + pos;
    updateSelectedBorder();
    var newsItem = getDetailedNews(newsId);
    if (newsItem != null) {
      updateNewsDetails(newsItem);
    } else {
      if (newsCache.length >= 5) {
        newsCache.shift();
      }
      Putpat.getNewsDetails(newsId, function(news) {
        newsCache.push(news);
        updateNewsDetails(news);
      });
    }
  }

  function scrollText(dir) {
    var top = 0;

    if (Device.moveTextWithMargin == false) {
      if ($("#news_content").css("top")) {
        var match = $("#news_content").css("top").match(/(.*)px/);
        if (match) {
          top = parseInt(match[1]);
        }
      }
    } else {
      if ($("#news_content").css("margin-top")) {
        var match = $("#news_content").css("margin-top").match(/(.*)px/);
        if (match) {
          top = parseInt(match[1]);
        }
      }
    }
    top += dir;

    if ((top*-1) + Positions.news.text_height > $("#news_content").height()) {
      top = Positions.news.text_height - $("#news_content").height();
    } else if (top > 0) {
      top = 0;
    }

    if (Device.moveTextWithMargin == false) {
      $("#news_content").css("top", top + "px");
    } else {
      $("#news_content").css("margin-top", top + "px");
    }

    var height = $("#news_content").height();
    var scrollHeight = (100 / (height / ((top * -1) / 100))) * Positions.news.scroll_height;
    $("#news_panel #news_scrollbar").css("margin-top", scrollHeight+"px");
  }

  function executeButton() {
    var regResult = selected.match(/news_box_(.*)/)

    var repeatKey = false;

    if (regResult) {
      Putpat.trackIvwUi();
      var pos = parseInt(regResult[1]);
      showNews(pos, newsItems[pos + startOffset].news.id);
    } else if (selected == "button_scroll_down") {
      scrollText(-30);
      repeatKey = true;
    } else if (selected == "button_scroll_up") {
      scrollText(30);
      repeatKey = true;
    } else if (selected == "button_close") {
      self.close();
    } else if (selected == "button_play") {
      Putpat.debug("button play pressed in news!");

      self.close();
      ControlBar.hide();
      var playlistName = getDetailedNews(newsItems[selectedNews].news.id).news.associated_playlist_name;
      Tracking.trackAction("user_action", "news_playlist_start", playlistName);
      ChannelView.startPlaylist(playlistName);
    } else if (selected == "news_left") {
      scrollNews(-1);
    } else if (selected == "news_right") {
      scrollNews(1);
    }
    return repeatKey;
  }

  function updateStartOffsetAndNews(dir, box) {
    startOffset += dir;
    updateNews();
    selectButton("news_box_"+box);
    var date = newsItems[startOffset + box].news.publishable_from.match(/(\d*)-(\d*)-(\d*)T.*/);
    $("#news_highlight_date").empty().append(date[3]+"."+date[2]+"."+date[1]);
    updateSelectedBorder();
  }

  function scrollNews(dir) {
    if (loadingNews) {
      return;
    }

    startOffset += dir;
    if (startOffset < 0) {
      startOffset = 0;
    }

    if (startOffset + MAX_NEWS < newsItems.length) {
      updateNews();
    } else if (!newsAllLoaded && !loadingNews) {
      loadMoreNews(function() {
        updateNews();
      });
    } else if (newsAllLoaded) {
      startOffset = newsItems.length - MAX_NEWS;
      updateNews();
    }
  }

  function moveSelectionHorizontally(dir) {
    var regResult = selected.match(/news_box_(.*)/);

    if (regResult) {
      var pos = parseInt(regResult[1]) + dir;
      if (pos < 0) {
        pos = 0;
        if (startOffset > 0) {
          updateStartOffsetAndNews(-1, pos);
        }
      } else if (pos >= MAX_NEWS) {
        pos = MAX_NEWS - 1;
        if (startOffset + MAX_NEWS + 1 < newsItems.length) {
          updateStartOffsetAndNews(1, pos);
        } else if (!newsAllLoaded && !loadingNews) {
          loadMoreNews(function() {
            if (startOffset + MAX_NEWS  + 1< newsItems.length) {
              updateStartOffsetAndNews(1, pos);
            }
          });
        }
      } else {
        updateStartOffsetAndNews(0, pos);
      }
    } else if ((selected == "button_scroll_up" || selected == "button_scroll_down")  && playlistActivated){
      selectButton("button_play");
    } else if (selected == "button_play" && scrollbarActivated) {
      selectButton("button_scroll_down");
    } else if (selected == "button_play") {
      if (dir < 0) {
        selectButton("button_close");
      } else {
        selectButton("news_box_"+selectedNewsBox);
      }
    } else if (selected == "news_left" && dir == +1) {
      selectButton("news_box_0");
    } else if (selected == "news_right" && dir == -1) {
      selectButton("news_box_4");
    }
  }

  function moveSelectionVertically(dir) {
    if (dir < 0) {
      if (selected.match(/news_(.*)/)) {
        if (playlistActivated) {
          selected = "button_play";
        } else if (scrollbarActivated) {
          selected = "button_scroll_down";
        } else {
          selected = "button_close";
        }
      } else if (selected == "button_scroll_down") {
        selected = "button_scroll_up";
      } else if (selected == "button_scroll_up") {
        selected = "button_close";
      } else if (selected == "button_close") {
         resetButtonState();
         selected = null;
         ControlBar.focus();
      } else if (selected == "button_play") {
        selected = "button_close";
      }
    } else {
      if (selected.match(/news_(.*)/)) {
         resetButtonState();
         selected = null;
         ControlBar.focus();
      } else if (selected == "button_close") {
        if (playlistActivated) {
          selected = "button_play";
        } else if (scrollbarActivated) {
          selected = "button_scroll_up";
        } else {
          selected = "news_box_" + selectedNewsBox;
        }
      } else if (selected == "button_scroll_up") {
        selected = "button_scroll_down";
      } else {
        selected = "news_box_" + selectedNewsBox;
      }
    }
    if (selected != null) {
      selectButton(selected);
    }
  }

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     var repeatKey = false;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             moveSelectionHorizontally(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_RIGHT:
             moveSelectionHorizontally(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_UP:
             moveSelectionVertically(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             moveSelectionVertically(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_ENTER:
             repeatKey = executeButton();
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             resetButtonState();
             ControlBar.focus();
             event.preventDefault();
             break
         case KEYS.KEY_RETURN:
             self.close();
             event.preventDefault();
             break;
     }

     if (repeatKey && ($("#news_panel").length > 0)) {
        keyboardRepeatTimer = setTimeout(function() {
          if (visible) {
            keyboardHandler(event);
          }
        }, 200);
     }
  }

  function keyboardUpHandler(event) {
    clearTimeout(keyboardRepeatTimer);
  }

  function updateNews() {
    var newsDiv = "";
    var removedEl = $("#news_panel #news_items").remove();
    for (var i = startOffset; i < startOffset + MAX_NEWS; i++) {
      var bNum = i - startOffset;
      newsDiv += "<div id='news_box_"+bNum+"' class='news_box news_box_"+bNum+"'>" +
            "<img class='image image_"+i+"' src='"+newsItems[i].news.news_image+"'/>" +
            "<div class='headline'>"+Utils.replaceWithHTMLEntities(newsItems[i].news.headline)+"</div>" +
            "<div class='subheadline'>"+Utils.replaceWithHTMLEntities(newsItems[i].news.sub_headline)+"</div>" +
        "</div>";
    }

    $("#news_panel").append(
      "<div id='news_items'>" +
      newsDiv +
      "</div>"
    );

    wireMouseEvents("news_box_0");
    wireMouseEvents("news_box_1");
    wireMouseEvents("news_box_2");
    wireMouseEvents("news_box_3");
    wireMouseEvents("news_box_4");

    if (!newsAllLoaded || (startOffset + MAX_NEWS < newsItems.length)) {
      $("#news_panel img#news_right").show();
    } else {
      $("#news_panel img#news_right").hide();
    }

    if (startOffset > 0) {
      $("#news_panel img#news_left").show();
    } else {
      $("#news_panel img#news_left").hide();
    }
  }

  function loadMoreNews(callback) {
    var removedEl = $("#news_panel #news_items").remove();
    loadingNews = true;
    Putpat.getNews(newsItems.length, function(news) {
      if (news.length < 5) {
        newsAllLoaded = true;
      }
      newsItems = newsItems.concat(news);
      loadingNews = false;
      callback();
    });
  }

  self.show = function(fullGraphics) {
    ChannelView.hideChannelLogo();

    $("#news_panel").remove();

    if (fullGraphics) {
      $("#putpat_view").append(
        "<div id='news_panel'>" +
          "<img class='background' src='" + Positions.size + "images/news/bg_news.jpg'/>" +
        "</div>"
      );
    }

    $("#news_panel").append(
      "<img id='button_close' class='news_button' src='"+Positions.size+"images/bt_close.png'/>" +
      "<div id='news_details'>" +
        "<div id='news_headline'></div>" +
        "<div id='news_flowtext'><div id='news_content'></div></div>" +
      "</div>" +
      "<div id='news_selected'>" +
        "<img id='news_selected_image' src='"+Positions.size + "images/news/news_item.png'/>" +
        "<div id='news_selected_date'>12.12.2011</div>"+
      "</div>"+
      "<div id='news_highlight'>" +
        "<img id='news_highlight_image' src='"+Positions.size + "images/news/news_item_hover.png'/>" +
        "<div id='news_highlight_date'>12.12.2011</div>"+
      "</div>"+
      "<img id='news_left' class='news_button' src='"+Positions.size + "images/news/more_items_left.png'/>" +
      "<img id='news_right' class='news_button' src='"+Positions.size + "images/news/more_items_right.png'/>" +
      "<div id='news_photo'>"+
        "<img id='news_photo_img'/><br style='clear:both'/>" + 
        "<div id='news_photo_copyright'></div>"+
      "</div>"+
      "<img id='button_play' class='news_button' src='"+Positions.size+"images/news/bt_play.png'/>" +
      "<img id='button_scroll_up' class='news_button' src='"+Positions.size + "images/bt_up.png'/>" +
      "<div id='news_scrollbar'></div>" +
      "<img id='button_scroll_down' class='news_button' src='"+Positions.size + "images/bt_down.png'/>"
    );

    wireMouseEvents("button_scroll_up");
    wireMouseEvents("button_scroll_down");
    wireMouseEvents("button_close");
    wireMouseEvents("button_play");
    wireMouseEvents("news_left");
    wireMouseEvents("news_right");

    selected = "news_box_0";
    startOffset = 0;
    loadMoreNews(function() {
      updateStartOffsetAndNews(0, 0);
      showNews(0, newsItems[startOffset].news.id);
    });

    MessageQueue.sendMessage("disable_bauchbinde");
    self.focus();
    visible = true;
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $("#"+button).mouseover(function() {
         ControlBar.unfocus();
         selectButton(this.id);
      });
      $("#"+button).click(function() {
        ControlBar.unfocus();
        if (selected != this.id) {
          selectButton(this.id);
        }
        executeButton(this.id);
      });
    }
  }

  self.hide = function() {
    MessageQueue.sendMessage("enable_bauchbinde");
    $("#news_panel").hide();
    $("#news_panel img").remove();
    ChannelView.showChannelLogo();
    visible = false;
  }

  self.close = function() {
    self.hide();
    ControlBar.currentScreenClosed();
  }

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
      keyboardUpHandler: keyboardUpHandler
    });
    if (selected == null) {
      selected = "news_box_" + selectedNewsBox;
    }
    selectButton(selected);
  }

  self.unfocus = function() {
    resetButtonState();
  }

  return self;
}();
