var SettingsPanel = function() {
  var self = {};

  var BUTTON_ROTATION = [
      'button_low',
      'button_medium',
      'button_close'
  ];

  var selected = null;

  function keyboardHandler(event) {
     var keycode = event.keyCode;

     switch(keycode) {
         case KEYS.KEY_LEFT:
             moveSelection(-1);
             event.preventDefault();
             break;
         case KEYS.KEY_RIGHT:
             moveSelection(+1);
             event.preventDefault();
             break;
         case KEYS.KEY_ENTER:
             executeButton();
             event.preventDefault();
             break;
         case KEYS.KEY_DOWN:
             resetButtonState();
             ControlBar.focus();
             event.preventDefault();
             break
         case KEYS.KEY_RETURN:
             self.close();
             event.preventDefault();
             break;
     }
  };

  function executeButton() {
    if (selected == "button_close") {
      self.close();
    } else if (selected == "button_qmenu") {
      setTimeout(function() {
        self.hide();
        ControlBar.hide();
        window.NetCastLaunchQMENU();
      }, 50);
    } else if (selected == "button_ratio") {
      setTimeout(function() {
        self.hide();
        ControlBar.hide();
        window.NetCastLaunchRATIO();
      }, 50);
    } else {
      var quality = selected.substring(7);
      if (Putpat.getCurrentQuality() != quality) {
        Tracking.trackAction("user_action", "change_quality");
        Putpat.setCurrentQuality(quality);
        resetButtonState();
        setTimeout(function() {
          self.hide();
          ControlBar.hide();
          ChannelView.switchToChannel(ChannelView.currentChannel, true, false);
        }, 150);
      } else {
        self.hide();
        ControlBar.hide();
      }
    }
  }

  function resetButtonState() {
    $.each(BUTTON_ROTATION, function(i, button_name) {
      $("#settings_panel #"+button_name).attr("src", $("#settings_panel #"+button_name).attr('src').replace("_hover.png", ".png"));
    });
  }

  function selectButton(button_name) {
    resetButtonState();

    selected = button_name;

    $("#settings_panel #"+button_name).attr('src', $("#settings_panel #"+button_name).attr('src').replace(".png", "_hover.png"));
  }

  function moveSelection(dir) {
    var newitem= BUTTON_ROTATION.indexOf(selected) + dir;

    if (newitem < 0) {
      newitem = BUTTON_ROTATION.length - 1;
    } else if (newitem >= BUTTON_ROTATION.length) {
      newitem = 0;
    }

    selectButton(BUTTON_ROTATION[newitem]);
  }

  function wireMouseEvents(button) {
    if (Device.hasMouse) {
      $("#"+button).mouseover(function() {
         ControlBar.unfocus();
         selectButton(this.id)
      });
      $("#"+button).click(function() {
        ControlBar.unfocus();
        if (selected != this.id) {
          selectButton(this.id)
        };
        executeButton(this.id);
      });
    }
  }

  self.show = function(all_graphics) {
    Tracking.trackAction("impression", "settings");

    $("#settings_panel").remove();
    $("#putpat_view").append(
      "<div id='settings_panel'>" +
         "<img class='background' src='" + Positions.size + "images/settings/bg_settings.jpg'/>" +
         "<img id='button_close'  src='"+Positions.size+"images/bt_close.png'/>" +
         "<img id='button_low' src='"+Positions.size+"images/settings/bt_low_unpressed.png'/>" +
         "<img id='button_medium' src='"+Positions.size+"images/settings/bt_high_unpressed.png'/>" +
      "</div>"
    );
    wireMouseEvents("button_close");
    wireMouseEvents("button_low");
    wireMouseEvents("button_medium");

    if (Device.showQEmu) {
      $("#settings_panel").append(
        "<img id='button_qmenu' src='"+Positions.size+"images/settings/bt_qmenu_unpressed.png'/>" +
        "<img id='button_ratio' src='"+Positions.size+"images/settings/bt_ratio_unpressed.png'/>"
      );
      wireMouseEvents("button_qmenu");
      wireMouseEvents("button_ratio");
      BUTTON_ROTATION = ["button_qmenu", "button_ratio"].concat(BUTTON_ROTATION);
    }

    selected = "button_" + Putpat.getCurrentQuality();
    if (Putpat.getCurrentQuality()== "medium") {
      $("#settings_panel #button_medium").attr('src', Positions.size+"images/settings/bt_high_pressed.png");
    } else {
      $("#settings_panel #button_low").attr('src', Positions.size+"images/settings/bt_low_pressed.png"); 
    }
    self.focus();
  }

  self.hide = function() {
    $("#settings_panel").hide();
    $("#settings_panel img").remove();
    ChannelView.showChannelLogo();
  }

  self.close = function() {
    self.hide();
    ControlBar.currentScreenClosed();
  }

  self.focus = function() {
    InputHandler.replaceKeyHandlers({
      keyboardHandler: keyboardHandler,
    });
    selectButton(selected);
  }

  self.unfocus = function() {
    resetButtonState();
  }

  return self;
}();
